# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 TU Wien.
# Copyright (C) 2022 CERN.
# Copyright (C) 2022 European Union.
#
# Invenio-Users-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""User groups resource."""

from flask import g, send_file
from flask_resources import resource_requestctx, response_handler, route
from invenio_records_resources.resources import RecordResource
from invenio_records_resources.resources.records.resource import (
    request_search_args,
    request_view_args,
)
from invenio_records_resources.resources.records.utils import search_preference


#
# Resource
#
class GroupsResource(RecordResource):
    """Resource for user groups."""

    def create_url_rules(self):
        """Create the URL rules for the user groups resource."""
        routes = self.config.routes
        return [
            route("GET", routes["list"], self.search),
            route("GET", routes["item"], self.read),
            route("GET", routes["item-avatar"], self.avatar),
        ]

    @request_search_args
    @request_view_args
    @response_handler(many=True)
    def search(self):
        """Perform a search over the groups."""
        hits = self.service.search(
            identity=g.identity,
            params=resource_requestctx.args,
            search_preference=search_preference(),
        )
        return hits.to_dict(), 200

    @request_view_args
    @response_handler()
    def read(self):
        """Read a group."""
        item = self.service.read(
            id_=resource_requestctx.view_args["id"],
            identity=g.identity,
        )
        return item.to_dict(), 200

    @request_view_args
    def avatar(self):
        """Get a groups's avatar."""
        avatar = self.service.read_avatar(
            name_=resource_requestctx.view_args["id"],
            identity=g.identity,
        )
        return send_file(
            avatar.bytes_io,
            mimetype=avatar.mimetype,
            as_attachment=False,
            download_name=avatar.name,
            etag=avatar.etag,
            last_modified=avatar.last_modified,
            max_age=86400 * 7,
        )



# -*- coding: utf-8 -*-
#
# Copyright (C) 2024-2025 CERN.
# Copyright (C) 2022 TU Wien.
#
# Invenio-Users-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Domains service tasks."""

import requests
from celery import shared_task
from flask import current_app
from invenio_accounts.models import Domain, DomainCategory, DomainStatus, User
from invenio_db import db
from invenio_search.engine import search

from ...proxies import current_domains_service, current_users_service


@shared_task(ignore_result=True)
def reindex_domains(domain_ids):
    """Reindex the given domains."""
    index = current_domains_service.record_cls.index
    if current_domains_service.indexer.exists(index):
        try:
            current_domains_service.indexer.bulk_index(domain_ids)
        except search.exceptions.ConflictError as e:
            current_app.logger.warn(f"Could not bulk-reindex groups: {e}")


@shared_task(ignore_result=True)
def delete_domains(domain_ids):
    """Delete domains from index."""
    index = current_domains_service.record_cls.index
    if current_domains_service.indexer.exists(index):
        try:
            current_domains_service.indexer.bulk_delete(domain_ids)
        except search.exceptions.ConflictError as e:
            current_app.logger.warn(f"Could not bulk-unindex groups: {e}")


@shared_task
def import_domain_blocklist(url, source_name):
    """Run import of domain blocklist.

    Marks domain as a spammer and blocks account registration from that domain.
    If domain already exists, it simply flags the domain, but does not block
    the domain.
    """
    spammer_category = DomainCategory.get("spammer")
    changed = set()

    for d in download_blocklist(url):
        if d.endswith("."):
            d = d[:-1]
        domain = Domain.query.filter_by(domain=d).one_or_none()
        if domain is None:
            current_app.logger.info(f"Creating {d}")
            try:
                domain = Domain.create(
                    d,
                    status=DomainStatus.blocked,
                    flagged=True,
                    flagged_source=source_name,
                    category=spammer_category.id,
                )
                db.session.commit()
                changed.add((domain.id, d))
            except Exception:
                current_app.logger.error(f"Error creating domain {d}")
                pass
        elif domain.flagged != True:
            current_app.logger.info(f"Updating ({domain.status}) {d}")
            domain.flagged = True
            domain.flagged_source = source_name
            db.session.commit()
            changed.add((domain.id, d))

    if len(changed) > 0:
        current_app.logger.info("Reindexing changed domains...")
        current_domains_service.indexer.bulk_index([x[0] for x in changed])

        current_app.logger.info("Reindexing users in changed domains...")
        for dummy, d in changed:
            users = db.session.query(User.id).filter(User.domain == d).yield_per(1000)
            current_users_service.indexer.bulk_index([u.id for u in users])


def download_blocklist(url):
    """Download domain blocklist from a given URL."""
    current_app.logger.info(f"Downloading {url}...")
    response = requests.get(url)
    if response.status_code != 200:
        current_app.logger.error(
            f"Failed to download {url}",
            extra={"url": url, "status_code": response.status_code},
        )
        return []

    # Parse file
    content = response.content.decode("utf-8")
    return {line.rstrip() for line in content.splitlines()}



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 CERN.
#
# Invenio-Users-Resources is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.
"""Users resources generic needs and permissions."""

from invenio_access import action_factory

USER_MANAGEMENT_ACTION_NAME = "administration-moderation"

user_management_action = action_factory(USER_MANAGEMENT_ACTION_NAME)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 TU Wien.
# Copyright (C) 2023-2025 Graz University of Technology.
# Copyright (C) 2024 Ubiquity Press.
#
# Invenio-Users-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""User and user group schemas."""

import re

from flask import current_app
from invenio_access.permissions import system_user_id
from invenio_accounts.models import DomainCategory
from invenio_accounts.profiles.schemas import (
    validate_locale,
    validate_timezone,
    validate_visibility,
)
from invenio_accounts.utils import DomainStatus
from invenio_i18n import lazy_gettext as _
from invenio_records_resources.services.records.schema import (
    BaseGhostSchema,
    BaseRecordSchema,
)
from marshmallow import (
    EXCLUDE,
    Schema,
    ValidationError,
    fields,
    post_load,
    pre_load,
    validate,
    validates_schema,
)
from marshmallow_utils.fields import Links, SanitizedUnicode, TZDateTime
from marshmallow_utils.permissions import FieldPermissionsMixin


class DomainInfoSchema(Schema):
    """Schema for domain info."""

    status = fields.String()
    tld = fields.String()
    flagged = fields.Boolean()


class IdentitiesSchema(Schema):
    """Schema for domain info."""

    github = fields.String()
    orcid = fields.String()


class UserPreferencesSchema(Schema):
    """Schema for user preferences."""

    visibility = fields.String(validate=validate_visibility)
    email_visibility = fields.String(validate=validate_visibility)
    locale = fields.String(validate=validate_locale)
    timezone = fields.String(validate=validate_timezone)


class UserProfileSchema(Schema):
    """Schema for user profiles."""

    full_name = fields.String()
    affiliations = fields.String()


def validate_username(value):
    """Validate username against accounts username regular expresion."""
    username_regex = current_app.config["ACCOUNTS_USERNAME_REGEX"]
    if not re.fullmatch(username_regex, value):
        raise ValidationError(str(current_app.config["ACCOUNTS_USERNAME_RULES_TEXT"]))


class UserSchema(BaseRecordSchema, FieldPermissionsMixin):
    """Schema for users."""

    field_dump_permissions = {
        "email": "read_email",
        "domain": "read_details",
        "created": "read_details",
        "updated": "read_details",
        "revision_id": "read_details",
        "active": "read_details",
        "status": "read_system_details",
        "visibility": "read_system_details",
        "confirmed": "read_details",
        "verified": "read_details",
        "blocked": "read_details",
        "preferences": "read_details",
        "domaininfo": "read_system_details",
        "blocked_at": "read_system_details",
        "verified_at": "read_system_details",
        "confirmed_at": "read_system_details",
        "current_login_at": "read_system_details",
    }

    # NOTE: API should only deliver users that are active & confirmed
    active = fields.Boolean()
    confirmed = fields.Boolean(dump_only=True)
    blocked = fields.Boolean(dump_only=True)
    verified = fields.Boolean(dump_only=True)
    status = fields.Str(dump_only=True)
    visibility = fields.Str(dump_only=True)
    is_current_user = fields.Method("is_self", dump_only=True)

    email = fields.Email(required=True)
    domain = fields.String()
    domaininfo = fields.Nested(DomainInfoSchema)
    identities = fields.Nested(IdentitiesSchema, dump_default={})
    username = fields.String(validate=validate_username)
    profile = fields.Dict()
    preferences = fields.Nested(UserPreferencesSchema)

    blocked_at = TZDateTime(dump_only=True)
    verified_at = TZDateTime(dump_only=True)
    confirmed_at = TZDateTime(dump_only=True)
    current_login_at = TZDateTime(dump_only=True)
    created = TZDateTime(dump_only=True)
    updated = TZDateTime(dump_only=True)

    def is_self(self, obj):
        """Determine if identity is the current identity."""
        current_identity = self.context["identity"]

        _id = obj.get("id") or obj.id

        return (
            _id is not None
            and current_identity.id is not None
            and str(_id) == str(current_identity.id)
        )


class GroupSchema(BaseRecordSchema):
    """Schema for user groups."""

    name = fields.String()
    title = fields.String()
    description = fields.String()
    provider = fields.String(dump_only=True)
    is_managed = fields.Boolean(dump_only=True)


class UserGhostSchema(BaseGhostSchema):
    """User ghost schema."""

    id = SanitizedUnicode(dump_only=True)
    profile = fields.Constant(
        {
            "full_name": _("Deleted user"),
        },
        dump_only=True,
    )
    username = fields.Constant(_("Deleted user"), dump_only=True)


class SystemUserSchema(BaseGhostSchema):
    """System user schema."""

    id = fields.Constant(system_user_id, dump_only=True)
    profile = fields.Constant(
        {
            "full_name": _("System"),
        },
        dump_only=True,
    )
    username = fields.Constant(_("System"), dump_only=True)
    email = fields.Constant("noreply@inveniosoftware.org", dump_only=True)


class NotificationPreferences(Schema):
    """Schema for notification preferences."""

    enabled = fields.Bool()


class DomainOrgSchema(Schema):
    """Schema for domain orgs."""

    id = fields.Integer(dump_only=True)
    pid = fields.String(validate=validate.Length(min=1, max=255), required=True)
    name = fields.String(validate=validate.Length(min=1, max=255), required=True)
    props = fields.Dict(
        keys=fields.String(required=True),
        values=fields.String(validate=validate.Length(max=255)),
    )
    is_parent = fields.Boolean(dump_only=True, dump_default=False)

    @validates_schema
    def validate_props(self, data, **kwargs):
        """Apply instance specific validation on props."""
        schema = current_app.config["USERS_RESOURCES_DOMAINS_ORG_SCHEMA"]
        props = data.get("props", {})
        if props:
            schema.load(props)


def validate_domain(value):
    """Domain validation."""
    # Basic validation - zenodo has some pretty funky domains so we are not too
    # strict here.
    if len(value) > 255:
        raise ValidationError("Length must be less than 255.")
    value = value.lower().strip()
    if "." not in value:
        raise ValidationError("Not a domain name.")
    prefix, tld = value.rsplit(".", 1)
    if tld == "":
        raise ValidationError("Not a domain name.")


class DomainSchema(Schema):
    """Schema for user groups."""

    id = fields.Str(dump_only=True)
    domain = fields.String(
        validate=validate_domain, required=True, metadata={"create_only": True}
    )
    tld = fields.String(dump_only=True)
    status = fields.Integer(dump_only=True)
    status_name = fields.String(
        validate=validate.OneOf([s.name for s in list(DomainStatus)]),
        load_default=DomainStatus.new.name,
    )
    category = fields.Integer(dump_only=True, metadata={"read_only": True})
    category_name = fields.String(validate=validate.Length(min=1, max=255))
    flagged = fields.Boolean(dump_default=False, metadata={"checked": False})
    flagged_source = fields.Str(validate=validate.Length(max=255), load_default="")
    org = fields.List(
        fields.Nested(DomainOrgSchema), dump_default=None, load_default=None
    )
    num_users = fields.Integer(dump_only=True)
    num_active = fields.Integer(dump_only=True)
    num_inactive = fields.Integer(dump_only=True)
    num_confirmed = fields.Integer(dump_only=True)
    num_verified = fields.Integer(dump_only=True)
    num_blocked = fields.Integer(dump_only=True)
    created = TZDateTime(dump_only=True)
    updated = TZDateTime(dump_only=True)
    links = Links(dump_only=True)

    class Meta:
        """Schema meta."""

        unknown = EXCLUDE

    @pre_load
    def preprocess(self, data, **kwargs):
        """Preprocess form data."""
        # Handle misbehaving clients.
        if "org" in data and data["org"] == "":
            del data["org"]
        return data

    @post_load
    def postprocess(self, data, **kwargs):
        """Process output data."""
        data["domain"] = data["domain"].lower().strip()
        data["domain"].strip()
        data["status_name"] = DomainStatus[data["status_name"]]
        data["status"] = data["status_name"].value
        if "category_name" in data:
            if data["category_name"] is None:
                data["category"] = None
            else:
                category = DomainCategory.get(data["category_name"])
                data["category"] = category.id
        if "org" in data:
            org = data["org"]
            if org is None or len(org) == 0:
                data["org"] = None
            else:
                # discard parent
                data["org"] = org[0]
        return data

    @validates_schema
    def validate_category(self, data, **kwargs):
        """Validate category data."""
        if "category_name" in data and data["category_name"] is not None:
            category = DomainCategory.get(data["category_name"])
            if category is None:
                raise ValidationError("Invalid category_name.")



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 TU Wien.
# Copyright (C) 2022 CERN.
# Copyright (C) 2024 Graz University of Technology.
# Copyright (C) 2024 Ubiquity Press.
#
# Invenio-Users-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""API classes for user and group management in Invenio."""

import unicodedata
from collections import namedtuple
from datetime import datetime

from flask import current_app
from invenio_accounts.models import Domain, User
from invenio_accounts.proxies import current_datastore
from invenio_db import db
from invenio_i18n import lazy_gettext as _
from invenio_records.dumpers import SearchDumper, SearchDumperExt
from invenio_records.dumpers.indexedat import IndexedAtDumperExt
from invenio_records.systemfields import ModelField
from invenio_records_resources.records.api import Record
from invenio_records_resources.records.systemfields import IndexField
from marshmallow import ValidationError
from sqlalchemy.exc import NoResultFound

from .dumpers import EmailFieldDumperExt
from .models import DomainAggregateModel, GroupAggregateModel, UserAggregateModel
from .systemfields import (
    AccountStatusField,
    AccountVisibilityField,
    DomainCategoryNameField,
    DomainField,
    DomainOrgField,
    DomainStatusNameField,
    IsNotNoneField,
    UserIdentitiesField,
)

EmulatedPID = namedtuple("EmulatedPID", ["pid_value"])
"""Emulated PID"""


class AggregatePID:
    """Helper emulate a PID field."""

    def __init__(self, pid_field):
        """Constructor."""
        self._pid_field = pid_field

    def __get__(self, record, owner=None):
        """Evaluate the property."""
        if record is None:
            return GetRecordResolver(owner)
        return EmulatedPID(record[self._pid_field])


class GetRecordResolver(object):
    """Resolver that simply uses get record."""

    def __init__(self, record_cls):
        """Initialize resolver."""
        self._record_cls = record_cls

    def resolve(self, pid_value):
        """Simply get the record."""
        return self._record_cls.get_record(pid_value)


class BaseAggregate(Record):
    """An aggregate of information about a user group/role."""

    metadata = None
    """Disabled metadata field from the base class."""

    def __getitem__(self, name):
        """Get a dict key item."""
        try:
            return getattr(self.model, name)
        except AttributeError:
            raise KeyError(name)

    def __repr__(self):
        """Create string representation."""
        return f"<{self.__class__.__name__}: {self.model.data}>"

    def __unicode__(self):
        """Create string representation."""
        return self.__repr__()

    @classmethod
    def from_model(cls, sa_model):
        """Create an aggregate from an SQL Alchemy model."""
        return cls({}, model=cls.model_cls(model_obj=sa_model))

    def _validate(self, *args, **kwargs):
        """Skip the validation."""
        pass

    def commit(self):
        """Update the aggregate data on commit."""
        # You can only commit if you have an underlying model object.
        if self.model._model_obj is None:
            raise ValueError(f"{self.__class__.__name__} not backed by a model.")
        if self.model._model_obj not in db.session:
            with db.session.begin_nested():
                # make sure we get an id assigned
                db.session.add(self.model._model_obj)
        # Basically re-parses the model object.
        model = self.model_cls(model_obj=self.model._model_obj)
        self.model = model
        return self


def _validate_user_data(user_data):
    """Validate the entered data for the user creation.

    This is a special case for validation done outside of the schema because it requires
    database queries that can significantly slow down serialization. We want to perform
    this validation upon account creation.
    Also, we can't let this fail naturaly at the DB level because it will happen during the
    `commit` state of the UOW and the feedback to the form can't be sent.
    """
    errors = {}
    username = user_data["username"]
    email = user_data["email"]
    # Check if Email exists already
    existing_email = db.session.query(User).filter_by(email=email).first()
    if existing_email:
        errors["email"] = [_("Email already used by another account.")]
    # Check if Username exists already
    existing_username = db.session.query(User).filter_by(username=username).first()
    if existing_username:
        errors["username"] = [_("Username already used by another account.")]
    if errors:
        raise ValidationError(errors)


class UserAggregate(BaseAggregate):
    """An aggregate of information about a user."""

    model_cls = UserAggregateModel
    """The model class for the request."""

    # NOTE: the "uuid" isn't a UUID but contains the same value as the "id"
    #       field, which is currently an integer for User objects!
    dumper = SearchDumper(
        extensions=[
            EmailFieldDumperExt("email"),
            IndexedAtDumperExt(),
        ],
        model_fields={
            "id": ("uuid", int),
        },
    )
    """Search dumper with configured extensions."""

    index = IndexField("users-user-v3.0.0", search_alias="users")
    """The search engine index to use."""

    id = ModelField("id", dump_type=int)
    """The user identifier."""

    active = ModelField("active", dump_type=bool)
    """Determine is user is active and can login."""

    # Profile fields
    username = ModelField("username", dump_type=str)
    """The user's email address."""

    email = ModelField("email", dump_type=str)
    """The user's email address."""

    domain = ModelField("domain", dump_type=str)
    """The domain of the users' email address."""

    profile = ModelField("profile", dump_type=dict)
    """The user's profile."""

    preferences = ModelField("preferences", dump_type=dict)
    """User preferences."""

    # Timestamps fields
    confirmed_at = ModelField("confirmed_at", dump_type=datetime)
    """Timestamp for when account was confirmed."""

    verified_at = ModelField("verified_at", dump_type=datetime)
    """Timestamp for when account was verified."""

    blocked_at = ModelField("blocked_at", dump_type=datetime)
    """Timestamp for when account was blocked."""

    current_login_at = ModelField("current_login_at", dump_type=datetime)
    """Timestamp for when account was blocked."""

    confirmed = IsNotNoneField("confirmed_at", index=True)
    """Boolean to determine if verified."""

    verified = IsNotNoneField("verified_at", index=True)
    """Boolean to determine if verified."""

    blocked = IsNotNoneField("blocked_at", index=True)
    """Boolean to determine if verified."""

    # Status fields
    status = AccountStatusField(index=True)
    """Combined account status attribute."""

    visibility = AccountVisibilityField(index=True)
    """Combined profile visibility attribute."""

    domaininfo = DomainField(use_cache=True, index=True)
    """Domain information."""

    identities = UserIdentitiesField("identities", use_cache=True, index=True)
    """User identities."""

    @property
    def avatar_chars(self):
        """Get avatar characters for user."""
        text = None
        if self.profile.get("full_name"):
            text = self.profile["full_name"]
        elif self.username:
            text = self.username
        else:
            alphabet = "ABCDEFGHIJKLMNOPQRSTUVXYZ"
            text = alphabet[self.id % len(alphabet)]

        return text[0].upper()

    @property
    def avatar_color(self):
        """Get avatar color for user."""
        colors = current_app.config["USERS_RESOURCES_AVATAR_COLORS"]
        return colors[self.id % len(colors)]

    @classmethod
    def create(cls, data, id_=None, validator=None, format_checker=None, **kwargs):
        """Create a new User and store it in the database."""
        try:
            # Check if email and  username already exists by another account.
            _validate_user_data(data)
            # Create User
            account_user = current_datastore.create_user(**data)
            return cls.from_model(account_user)
        except ValidationError:
            raise
        except Exception as e:
            raise ValidationError(message=f"Unexpected Issue: {str(e)}", data=data)

    def verify(self):
        """Activates the current user.

        Activation of the user is proxied through the datastore.
        """
        user = self.model.model_obj
        if user is None:
            return False
        return current_datastore.verify_user(user)

    def block(self):
        """Blocks a user."""
        user = self.model.model_obj
        if user is None:
            return False
        return current_datastore.block_user(user)

    def activate(self):
        """Activate a previously deactivated user."""
        user = self.model.model_obj
        if user is None:
            return False
        return current_datastore.activate_user(user)

    def deactivate(self):
        """Deactivates the current user."""
        user = self.model.model_obj
        if user is None:
            return False
        return current_datastore.deactivate_user(user)

    @classmethod
    def get_record(cls, id_):
        """Get the user via the specified ID."""
        with db.session.no_autoflush:
            # Notifications builders right now manage to pass "system" as an
            # id when they try use the ServiceResultResolvers on
            # {'user': 'system'} which results in a database transaction being
            # rolled back when quering on an integer id column with a string.
            if id_ == "system":
                return None

            user = current_datastore.get_user_by_id(id_)
        if user is None:
            return None

        with db.session.no_autoflush:
            return cls.from_model(user)


class GroupAggregate(BaseAggregate):
    """An aggregate of information about a user group/role."""

    model_cls = GroupAggregateModel
    """The model class for the user group aggregate."""

    # NOTE: the "uuid" isn't a UUID but contains the same value as the "id"
    #       field, which is currently a str for Role objects (role.name)!
    dumper = SearchDumper(extensions=[], model_fields={"id": ("uuid", str)})
    """Search index dumper."""

    index = IndexField("groups-group-v2.0.0", search_alias="groups")
    """The search engine index to use."""

    id = ModelField("id", dump_type=str)
    """ID of group."""

    name = ModelField("name", dump_type=str)
    """The group's name."""

    description = ModelField("description", dump_type=str)
    """The group's description."""

    is_managed = ModelField("is_managed", dump_type=bool)
    """If the group is managed manually."""

    @property
    def avatar_chars(self):
        """Get avatar characters for user."""
        return self.id[0].upper()

    @property
    def avatar_color(self):
        """Get avatar color for user."""
        colors = current_app.config["USERS_RESOURCES_AVATAR_COLORS"]
        normalized_group_initial = unicodedata.normalize("NFKD", self.id[0]).encode(
            "ascii", "ignore"
        )
        return colors[int(normalized_group_initial, base=36) % len(colors)]

    @classmethod
    def get_record(cls, id_):
        """Get the user group via the specified ID."""
        # TODO how do we want to resolve the roles? via ID or name?
        with db.session.no_autoflush:
            role = db.session.get(current_datastore.role_model, id_)
            if role is None:
                return None
            return cls.from_model(role)

    @classmethod
    def get_record_by_name(cls, name):
        """Get the user group via the specified ID."""
        # TODO how do we want to resolve the roles? via ID or name?
        with db.session.no_autoflush:
            role = (
                db.session.query(current_datastore.role_model)
                .filter_by(name=name)
                .one_or_none()
            )
            if role is None:
                return None
            return cls.from_model(role)


class OrgNameDumperExt(SearchDumperExt):
    """Custom fields dumper extension."""

    def dump(self, record, data):
        """Dump for faceting."""
        org = data.get("org", None)
        if org and len(org) > 0:
            data["org_names"] = [o["name"] for o in org]

    def load(self, data, record_cls):
        """Remove data from object."""
        data.pop("org_names", None)


class DomainAggregate(BaseAggregate):
    """An aggregate of information about a user."""

    model_cls = DomainAggregateModel
    """The model class for the request."""

    # NOTE: the "uuid" isn't a UUID but contains the same value as the "id"
    #       field, which is currently an integer for User objects!
    dumper = SearchDumper(
        extensions=[
            IndexedAtDumperExt(),
            OrgNameDumperExt(),
        ],
        model_fields={
            "id": ("uuid", int),
        },
    )
    """Search dumper with configured extensions."""

    index = IndexField("domains-domain-v1.0.0", search_alias="domains")
    """The search engine index to use."""

    pid = AggregatePID("domain")
    """Needed to emulate pid access."""

    id = ModelField("id", dump_type=int)
    """The user identifier."""

    domain = ModelField("domain", dump_type=str)
    """The domain of the users' email address."""

    tld = ModelField("tld", dump_type=str)
    """Top level domain."""

    status = ModelField("status", dump_type=int)
    """Domain status."""

    status_name = DomainStatusNameField(index=True)
    """Domain status name."""

    flagged = ModelField("flagged", dump_type=bool)
    """Flagged."""

    flagged_source = ModelField("flagged_source", dump_type=str)
    """Source of flagging."""

    category = ModelField("category", dump_type=int)
    """Domain category."""

    category_name = DomainCategoryNameField(use_cache=True, index=True)
    """Domain category name."""

    org_id = ModelField("org_id", dump_type=int)
    """Number of users."""

    org = DomainOrgField("org", use_cache=True, index=True)
    """Organization behind the domain."""

    num_users = ModelField("num_users", dump_type=int)
    """Number of users."""

    num_active = ModelField("num_active", dump_type=int)
    """Number of active users."""

    num_inactive = ModelField("num_inactive", dump_type=int)
    """Number of inactive users."""

    num_confirmed = ModelField("num_confirmed", dump_type=int)
    """Number of confirmed users."""

    num_verified = ModelField("num_verified", dump_type=int)
    """Number of verified users."""

    num_blocked = ModelField("num_blocked", dump_type=int)
    """Number of blocked users."""

    @classmethod
    def get_record(cls, id_):
        """Get the user via the specified ID."""
        with db.session.no_autoflush:
            domain = current_datastore.find_domain(id_)
        if domain is None:
            raise NoResultFound()
        return cls.from_model(domain)

    @classmethod
    def create(cls, data, id_=None, **kwargs):
        """Create a domain."""
        return DomainAggregate(data, model=DomainAggregateModel(model_obj=Domain()))

    def delete(self, force=True):
        """Delete the domain."""
        db.session.delete(self.model.model_obj)



# -*- coding: utf-8 -*-
#
# Copyright (C)      2023 Graz University of Technology.
#
# Invenio-Users-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Forms for user resources."""

from flask_wtf import FlaskForm
from invenio_i18n import lazy_gettext as _
from wtforms import BooleanField

from invenio_users_resources.models import NotificationPreferencesProxy


class NotificationsForm(FlaskForm):
    """Form for editing user notification preferences."""

    profile_proxy_cls = NotificationPreferencesProxy

    enabled = BooleanField(
        _("Notify me"),
        description=_("Turn on to enable notifications for relevant events."),
    )

    def process(self, formdata=None, obj=None, data=None, extra_filters=None, **kwargs):
        """Build a proxy around the object."""
        if obj is not None:
            obj = self.profile_proxy_cls(obj)
        super().process(
            formdata=formdata, obj=obj, data=data, extra_filters=extra_filters, **kwargs
        )

    def populate_obj(self, user):
        """Populates the obj."""
        user = self.profile_proxy_cls(user)
        super().populate_obj(user)
